@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # This file is part of Xpra.
-# Copyright (C) 2018-2019 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2018 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -45,31 +45,31 @@ class ChildCommandMixinTest(ServerMixinTest):
         opts.exec_wrapper = None
         opts.start_env = []
         opts.source_start = []
-        #pynotify can cause crashes,
-        #probably due to threading issues?
+
+        # pynotify can cause crashes,
+        # probably due to threading issues?
+
         def noop():
             pass
+
         def _ChildCommandServer():
-            from gi.repository import GLib  # @UnresolvedImport
-            def idle_add(ccs, *args):
-                return GLib.idle_add(*args)
-            child_command.ChildCommandServer.idle_add = idle_add
             ccs = child_command.ChildCommandServer()
             ccs.setup_menu_watcher = noop
             return ccs
+
         self._test_mixin_class(_ChildCommandServer, opts)
         if not POSIX:
             return
-        #test creating a temp file:
+        # test creating a temp file:
         import tempfile
         tmpfile = os.path.join(tempfile.gettempdir(), "xpra-test-start-command-%s" % os.getpid())
         assert not os.path.exists(tmpfile)
-        command = (b"touch", tmpfile.encode("utf8"))
+        command = ("touch", tmpfile)
         with silence_info(child_command):
-            self.handle_packet(("start-command", b"test", command, True))
+            self.handle_packet(("start-command", "test", command, True))
         time.sleep(1)
         info = self.mixin.get_info(self.protocol)
-        commands = info.get("commands")
+        commands = info.get("command")
         assert commands
         proc_info = commands.get(0)
         assert proc_info
@@ -77,26 +77,26 @@ class ChildCommandMixinTest(ServerMixinTest):
         assert pid
         assert os.path.exists(tmpfile)
         os.unlink(tmpfile)
-        #test signals:
+        # test signals:
         with silence_info(child_command):
             self.handle_packet(("start-command", "sleep", "sleep 10", True))
         time.sleep(1)
         info = self.mixin.get_info(self.protocol)
-        commands = info.get("commands")
+        commands = info.get("command")
         assert commands
         proc_info = commands.get(1)
         assert proc_info
         pid = proc_info.get("pid")
         assert pid
-        assert proc_info.get("name")=="sleep"
+        assert proc_info.get("name") == "sleep"
         assert proc_info.get("dead") is False
-        #send it a SIGINT:
+        # send it a SIGINT:
         with silence_info(child_command):
             self.handle_packet(("command-signal", pid, "SIGINT"))
         time.sleep(1)
         self.mixin.child_reaper.poll()
         info = self.mixin.get_info(self.protocol)
-        commands = info.get("commands")
+        commands = info.get("command")
         assert commands
         proc_info = commands.get(1)
         assert proc_info.get("dead") is True
