@@ -1,6 +1,6 @@
 # This file is part of Xpra.
-# Copyright (C) 2010-2024 Antoine Martin <antoine@xpra.org>
-# Copyright (C) 2008, 2010 Nathaniel Smith <njs@pobox.com>
+# Copyright (C) 2010 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -8,13 +8,14 @@ import os.path
 import sys
 from time import monotonic
 from typing import Any
+from collections.abc import Sequence
 
 from xpra.util.objects import typedict
-from xpra.util.str_fn import nonl, csv, ellipsizer, repr_ellipsized, sorted_nicely, bytestostr, hexstr
+from xpra.util.str_fn import csv, Ellipsizer, repr_ellipsized, sorted_nicely, bytestostr, hexstr
 from xpra.util.env import envint, first_time
-from xpra.common import ConnectionMessage, disconnect_is_an_error
+from xpra.common import ConnectionMessage, disconnect_is_an_error, noop
 from xpra.os_util import gi_import, get_hex_uuid, POSIX, OSX
-from xpra.util.io import stderr_print
+from xpra.util.io import stderr_print, load_binary_file
 from xpra.net.common import PacketType
 from xpra.util.stats import std_unit
 from xpra.client.base.client import EXTRA_TIMEOUT
@@ -52,8 +53,9 @@ class CommandConnectClient(GObjectXpraClient):
         # for newer versions, it is easier:
         self.hello_extra["wants"] = []
 
-    def setup_connection(self, conn):
-        protocol = super().setup_connection(conn)
+    def make_protocol(self, conn):
+        self.add_packet_handler("setting-change", noop)
+        protocol = super().make_protocol(conn)
         if conn.timeout > 0:
             self.command_timeout = GLib.timeout_add((conn.timeout + self.COMMAND_TIMEOUT) * 1000, self.timeout)
         return protocol
@@ -87,14 +89,15 @@ class CommandConnectClient(GObjectXpraClient):
         # don't bother parsing the network caps:
         # * it could cause errors if the caps are missing
         # * we don't care about sending anything back after hello
-        log("server_capabilities: %s", ellipsizer(caps))
+        log("server_capabilities: %s", Ellipsizer(caps))
         log("protocol state: %s", self._protocol.save_state())
+        self.init_authenticated_packet_handlers()
         self.cancel_command_timeout()
         self.do_command(caps)
         return True
 
     def do_command(self, caps: typedict) -> None:
-        raise NotImplementedError()
+        raise NotImplementedError(f"no do_command method in {self!r}")
 
 
 class SendCommandConnectClient(CommandConnectClient):
@@ -120,15 +123,13 @@ class HelloRequestClient(SendCommandConnectClient):
     """
 
     def make_hello_base(self) -> dict[str, Any]:
-        caps = super().make_hello_base()
-        caps.update(self.hello_request())
-        return caps
+        return super().make_hello_base() | self.hello_request()
 
     def timeout(self, *_args) -> None:
         self.warn_and_quit(ExitCode.TIMEOUT, "timeout: server did not disconnect us")
 
     def hello_request(self):  # pragma: no cover
-        raise NotImplementedError()
+        raise NotImplementedError(f"`hello_request` not implemented in {self!r}")
 
     def do_command(self, caps: typedict) -> None:
         self.quit(ExitCode.OK)
@@ -137,7 +138,7 @@ class HelloRequestClient(SendCommandConnectClient):
         # overridden method, so we can avoid printing a warning,
         # we haven't received the hello back from the server
         # but that's fine for a request client
-        info = tuple(nonl(x) for x in packet[1:])
+        info = tuple(str(x) for x in packet[1:])
         reason = info[0]
         if disconnect_is_an_error(reason):
             self.server_disconnect_warning(*info)
@@ -147,7 +148,7 @@ class HelloRequestClient(SendCommandConnectClient):
             self.server_disconnect(*info)
 
 
-class ScreenshotXpraClient(CommandConnectClient):
+class ScreenshotXpraClient(HelloRequestClient):
     """ This client does one thing only:
         it sends the hello packet with a screenshot request
         and exits when the resulting image is received (or timedout)
@@ -156,7 +157,14 @@ class ScreenshotXpraClient(CommandConnectClient):
     def __init__(self, opts, screenshot_filename):
         self.screenshot_filename = screenshot_filename
         super().__init__(opts)
-        self.hello_extra["request"] = "screenshot"
+
+    def hello_request(self) -> dict[str, Any]:
+        return {
+            "request": "screenshot",
+        }
+
+    def do_command(self, caps: typedict) -> None:
+        self.send("screenshot")
 
     def timeout(self, *_args) -> None:
         self.warn_and_quit(ExitCode.TIMEOUT, "timeout: did not receive the screenshot")
@@ -180,7 +188,7 @@ class ScreenshotXpraClient(CommandConnectClient):
 
     def init_packet_handlers(self) -> None:
         super().init_packet_handlers()
-        self._ui_packet_handlers["screenshot"] = self._process_screenshot
+        self.add_packets("screenshot")
 
 
 class InfoXpraClient(CommandConnectClient):
@@ -203,7 +211,7 @@ class InfoXpraClient(CommandConnectClient):
         def print_fn(s) -> None:
             sys.stdout.write(f"{s}\n")
 
-        def prettify(k, v) -> str:
+        def prettify(k, v):
             ks = str(k)
             if ks.endswith("xid"):
                 try:
@@ -220,8 +228,10 @@ class InfoXpraClient(CommandConnectClient):
                 eltypes = set(type(x) for x in v)
                 if len(eltypes) == 1 and tuple(eltypes)[0] in (int, bool, float):
                     return str(v)
-                return str(type(v)(prettify(k, x) for x in v))
-            return str(v)
+                return type(v)(prettify(k, x) for x in v)
+            elif isinstance(v, str) and v.isprintable():
+                return v
+            return repr(v)
 
         def print_dict(d: dict, path=""):
             for k in sorted_nicely(d.keys()):
@@ -232,7 +242,7 @@ class InfoXpraClient(CommandConnectClient):
                     print_dict(v, kpath)
                     continue
                 fv = prettify(k, v)
-                print_fn(f"{kpath}={nonl(fv)}")
+                print_fn(f"{kpath}={fv}")
 
         exit_code = ExitCode.OK
         try:
@@ -319,8 +329,7 @@ class MonitorXpraClient(SendCommandConnectClient):
 
     def init_packet_handlers(self) -> None:
         super().init_packet_handlers()
-        self._packet_handlers["server-event"] = self._process_server_event
-        self._packet_handlers["ping"] = self._process_ping
+        self.add_packets("server-event", "ping")
 
     def _process_ping(self, packet: PacketType) -> None:
         echotime = packet[1]
@@ -368,21 +377,21 @@ class InfoTimerClient(MonitorXpraClient):
 
     def do_command(self, caps: typedict) -> None:
         self.send_info_request()
-        self.timeout_add(self.REFRESH_RATE * 1000, self.send_info_request)
+        GLib.timeout_add(self.REFRESH_RATE * 1000, self.send_info_request)
 
-    def send_info_request(self, *categories) -> bool:
+    def send_info_request(self, *categories: str) -> bool:
         self.log("send_info_request%s" % (categories,))
         if not self.info_request_pending:
             self.info_request_pending = True
             window_ids = ()  # no longer used or supported by servers
             self.send("info-request", [self.uuid], window_ids, categories)
         if not self.info_timer:
-            self.info_timer = self.timeout_add((self.REFRESH_RATE + 2) * 1000, self.info_timeout)
+            self.info_timer = GLib.timeout_add((self.REFRESH_RATE + 2) * 1000, self.info_timeout)
         return True
 
     def init_packet_handlers(self) -> None:
         MonitorXpraClient.init_packet_handlers(self)
-        self.add_packet_handler("info-response", self._process_info_response, False)
+        self.add_packets("info-response")
 
     def _process_server_event(self, packet: PacketType) -> None:
         self.log("server event: %s" % (packet,))
@@ -402,7 +411,7 @@ class InfoTimerClient(MonitorXpraClient):
         it = self.info_timer
         if it:
             self.info_timer = 0
-            self.source_remove(it)
+            GLib.source_remove(it)
 
     def info_timeout(self) -> bool:
         self.log("info timeout")
@@ -440,7 +449,7 @@ class ShellXpraClient(SendCommandConnectClient):
         siw = self.stdin_io_watch
         if siw:
             self.stdin_io_watch = None
-            self.source_remove(siw)
+            GLib.source_remove(siw)
         super().cleanup()
 
     def do_command(self, caps: typedict) -> None:
@@ -482,14 +491,14 @@ class ShellXpraClient(SendCommandConnectClient):
 
     def init_packet_handlers(self) -> None:
         super().init_packet_handlers()
-        self._packet_handlers["shell-reply"] = self._process_shell_reply
-        self._packet_handlers["ping"] = self._process_ping
+        self.add_packets("shell-reply", "ping")
 
     def _process_ping(self, packet: PacketType) -> None:
         echotime = packet[1]
         self.send("ping_echo", echotime, 0, 0, 0, -1)
 
-    def _process_shell_reply(self, packet: PacketType) -> None:
+    @staticmethod
+    def _process_shell_reply(packet: PacketType) -> None:
         fd = int(packet[1])
         message = packet[2]
         if fd == 1:
@@ -537,7 +546,7 @@ class ControlXpraClient(CommandConnectClient):
     """ Allows us to send commands to a server.
     """
 
-    def set_command_args(self, command) -> None:
+    def set_command_args(self, command: Sequence[str]) -> None:
         self.command = command
 
     def timeout(self, *_args) -> None:
@@ -545,7 +554,7 @@ class ControlXpraClient(CommandConnectClient):
 
     def do_command(self, caps: typedict) -> None:
         cr = caps.tupleget("command_response")
-        if cr is None:
+        if not cr or len(cr) < 2:
             self.warn_and_quit(ExitCode.UNSUPPORTED, "server does not support control command")
             return
         code = int(cr[0])
@@ -565,18 +574,17 @@ class ControlXpraClient(CommandConnectClient):
 
 
 class PrintClient(SendCommandConnectClient):
-    """ Allows us to send a file to the server for printing.
+    """
+    Allows us to send a file to the server for printing.
     """
 
-    def set_command_args(self, command) -> None:
+    def set_command_args(self, command: Sequence[str]) -> None:
         log("set_command_args(%s)", command)
         self.filename = command[0]
         # print command arguments:
         # filename, file_data, mimetype, source_uuid, title, printer, no_copies, print_options_str = packet[1:9]
         self.command = command[1:]
 
-        # TODO: load as needed...
-
         if self.filename == "-":
             # replace with filename proposed
             self.filename = command[2]
@@ -585,16 +593,15 @@ class PrintClient(SendCommandConnectClient):
                 self.file_data = stdin_binary.read()
             log("read %i bytes from stdin", len(self.file_data))
         else:
-            if not self.check_file_size(os.path.getsize(self.filename)):
+            if not self.validate_file_size(os.path.getsize(self.filename)):
                 return
-            from xpra.util.io import load_binary_file
             self.file_data = load_binary_file(self.filename)
             log("read %i bytes from %s", len(self.file_data), self.filename)
-        if not self.check_file_size(len(self.file_data)):
+        if not self.validate_file_size(len(self.file_data)):
             return
         assert self.file_data, "no data found for '%s'" % self.filename
 
-    def check_file_size(self, size: int) -> bool:
+    def validate_file_size(self, size: int) -> bool:
         if size <= self.file_size_limit:
             return True
         self.warn_and_quit(ExitCode.FILE_TOO_BIG,
@@ -619,7 +626,7 @@ class PrintClient(SendCommandConnectClient):
         blob = Compressed("print", self.file_data)
         self.send("print", self.filename, blob, *self.command)
         log("print: sending %s as %s for printing", self.filename, blob)
-        self.idle_add(self.send, "disconnect", ConnectionMessage.DONE.value, "detaching")
+        GLib.idle_add(self.send, "disconnect", ConnectionMessage.DONE.value, "detaching")
 
     def make_hello(self) -> dict[str, Any]:
         capabilities = super().make_hello()
@@ -640,7 +647,7 @@ class ExitXpraClient(HelloRequestClient):
         }
 
     def do_command(self, caps: typedict) -> None:
-        self.idle_add(self.send,
+        GLib.idle_add(self.send,
                       "exit-server",
                       os.environ.get("XPRA_EXIT_MESSAGE", ConnectionMessage.SERVER_EXIT.value)
                       )
@@ -662,8 +669,8 @@ class StopXpraClient(HelloRequestClient):
             return
         # with newer (v5) servers, the "stop" request should have done it,
         # but let's try harder for older servers:
-        self.timeout_add(1000, self.send_shutdown_server)
-        # self.idle_add(self.send_shutdown_server)
+        GLib.timeout_add(1000, self.send_shutdown_server)
+        # GLib.idle_add(self.send_shutdown_server)
         # not exiting the client here,
         # the server should send us the shutdown disconnection message anyway
         # and if not, we will then hit the timeout to tell us something went wrong
@@ -678,7 +685,7 @@ class DetachXpraClient(HelloRequestClient):
         }
 
     def do_command(self, caps: typedict) -> None:
-        self.idle_add(self.send, "disconnect", ConnectionMessage.DONE.value, "detaching")
+        GLib.idle_add(self.send, "disconnect", ConnectionMessage.DONE.value, "detaching")
         # not exiting the client here,
         # the server should disconnect us with the response
 
@@ -709,7 +716,7 @@ class RequestStartClient(HelloRequestClient):
             # this can be called again if we receive a challenge,
             # but only print this message once:
             errwrite("requesting new session, please wait")
-        self.timeout_add(1 * 1000, self.dots)
+        GLib.timeout_add(1 * 1000, self.dots)
         return {
             "start-new-session": self.start_new_session,
             # tells proxy servers we don't want to connect to the real / new instance:
