@@ -1,5 +1,5 @@
 # This file is part of Xpra.
-# Copyright (C) 2018-2024 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2018 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -15,7 +15,7 @@ from Cocoa import (
     NSOpenGLPFAOpenGLProfile, NSOpenGLProfileVersion3_2Core,
 )
 from xpra.gtk.window import GDKWindow
-from xpra.client.gl.check import check_PyOpenGL_support
+from xpra.opengl.check import check_PyOpenGL_support
 from xpra.platform.darwin.gdk3_bindings import get_nsview_ptr, enable_transparency, get_backing_scale_factor
 from xpra.util.env import envbool
 from xpra.os_util import gi_import
