@@ -1,5 +1,5 @@
 # This file is part of Xpra.
-# Copyright (C) 2011-2024 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2011 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -20,7 +20,7 @@ from xpra.client.gtk3.tray_menu import (
 )
 from xpra.platform.paths import get_icon
 from xpra.platform.darwin import get_OSXApplication
-from xpra.client.gui import features
+from xpra.client.base import features
 from xpra.log import Logger
 
 Gtk = gi_import("Gtk")
@@ -337,7 +337,7 @@ class OSXMenuHelper(GTKTrayMenu):
         self._clipboard_change_pending = False
         return label
 
-    def set_clipboard_menu(self, _clipboard_menu):
+    def set_clipboard_menu(self, clipboard_menu) -> None:
         # find the menu item matching the current settings,
         # and select it
         try:
@@ -346,11 +346,13 @@ class OSXMenuHelper(GTKTrayMenu):
             if len(ch._local_to_remote) == 1:
                 rc_setting = tuple(ch._local_to_remote.values())[0]
             label = CLIPBOARD_NAME_TO_LABEL.get(rc_setting)
+            clipboardlog(f"set_clipboard_menu(%s) setting={rc_setting!r}, {label=}", clipboard_menu)
             self.select_clipboard_menu_option(None, label, CLIPBOARD_LABELS)
-        except Exception:
+        except RuntimeError:
             clipboardlog("failed to select remote clipboard option in menu", exc_info=True)
-        direction_label = CLIPBOARD_DIRECTION_NAME_TO_LABEL.get(self.client.client_clipboard_direction, "Disabled")
-        clipboardlog("direction(%s)=%s", self.client.client_clipboard_direction, direction_label)
+        direction = self.client.client_clipboard_direction
+        direction_label = CLIPBOARD_DIRECTION_NAME_TO_LABEL.get(direction, "Disabled")
+        clipboardlog("direction(%s)=%s", direction, direction_label)
         self.select_clipboard_menu_option(None, direction_label, CLIPBOARD_DIRECTION_LABELS)
 
     # these methods are called by the superclass,
