@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # This file is part of Xpra.
-# Copyright (C) 2018-2024 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2018 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -14,6 +14,7 @@ import sys
 import glob
 from time import monotonic
 from contextlib import nullcontext
+from threading import RLock
 from typing import Any
 from collections.abc import Generator, Sequence
 
@@ -24,9 +25,11 @@ from xpra.log import Logger
 
 log = Logger("exec", "menu")
 
-LOAD_FROM_RESOURCES: bool = envbool("XPRA_XDG_LOAD_FROM_RESOURCES", True)
-LOAD_FROM_PIXMAPS: bool = envbool("XPRA_XDG_LOAD_FROM_PIXMAPS", True)
-LOAD_FROM_THEME: bool = envbool("XPRA_XDG_LOAD_FROM_THEME", True)
+ENABLED: bool = envbool("XPRA_XDG", True)
+
+LOAD_FROM_RESOURCES: bool = envbool("XPRA_XDG_LOAD_FROM_RESOURCES", ENABLED)
+LOAD_FROM_PIXMAPS: bool = envbool("XPRA_XDG_LOAD_FROM_PIXMAPS", ENABLED)
+LOAD_FROM_THEME: bool = envbool("XPRA_XDG_LOAD_FROM_THEME", ENABLED)
 LOAD_GLOB: bool = envbool("XPRA_XDG_LOAD_GLOB", False)
 
 EXPORT_ICONS: bool = envbool("XPRA_XDG_EXPORT_ICONS", True)
@@ -185,7 +188,7 @@ def load_entry_icon(props: dict):
     return props
 
 
-def find_icon(*names) -> str:
+def find_icon(*names: str) -> str:
     if not EXPORT_ICONS:
         return ""
     return find_resources_icon(*names) or \
@@ -194,7 +197,7 @@ def find_icon(*names) -> str:
         find_glob_icon(*names, category="apps")
 
 
-def find_resources_icon(*names) -> str:
+def find_resources_icon(*names: str) -> str:
     if not LOAD_FROM_RESOURCES:
         return ""
     # loads the icon from our own list of icons:
@@ -205,7 +208,7 @@ def find_resources_icon(*names) -> str:
     return ""
 
 
-def find_pixmap_icon(*names) -> str:
+def find_pixmap_icon(*names: str) -> str:
     if not LOAD_FROM_PIXMAPS:
         return ""
     pixmaps_dirs = [d + '/icons' for d in os.environ.get("XDG_DATA_DIRS", "").split(":") if d]
@@ -223,7 +226,10 @@ def find_pixmap_icon(*names) -> str:
     return ""
 
 
-def find_theme_icon(*names) -> str:
+ic_lock = RLock()
+
+
+def find_theme_icon(*names: str) -> str:
     if not LOAD_FROM_THEME:
         return ""
     if not (IconTheme and Config and themes):
@@ -231,21 +237,39 @@ def find_theme_icon(*names) -> str:
     size = Config.icon_size or 32
     for name in names:
         for theme in themes.values():
-            try:
-                fn = IconTheme.LookupIcon(name, size, theme=theme, extensions=EXTENSIONS)
-                if fn and os.path.exists(fn):
-                    return fn
-            except TypeError as e:
-                log(f"find_theme_icon({names}) error on {name=}, {size=}", exc_info=True)
-                if first_time("xdg-icon-lookup"):
-                    log.warn(f"Warning: icon loop failure for {name} in {theme}")
-                    log.warn(f" {e}")
-                    log.warn(" this is likely to be this bug in pyxdg:")
-                    log.warn(" https://github.com/takluyver/pyxdg/pull/20")
+            with ic_lock:
+                try:
+                    fn = IconTheme.LookupIcon(name, size, theme=theme, extensions=EXTENSIONS)
+                    if fn and os.path.exists(fn):
+                        return fn
+                except TypeError as e:
+                    log(f"find_theme_icon({names}) error on {name=}, {size=}", exc_info=True)
+                    if first_time("xdg-icon-lookup"):
+                        log.warn(f"Warning: icon loop failure for {name} in {theme}")
+                        log.warn(f" {e}")
+                        log.warn(" this is likely to be this bug in pyxdg:")
+                        log.warn(" https://github.com/takluyver/pyxdg/pull/20")
     return ""
 
 
-def find_glob_icon(*names, category: str = "categories"):
+def add_entry_icon(entry_props: dict[str, Any], category: str):
+    """ adds 'IconData' and 'IconType' to `entry_props` if we find a matching icon """
+    if not entry_props:
+        return
+    names = []
+    for prop in ("Name", "GenericName", "Exec", "TryExec"):
+        value = entry_props.get(prop, "")
+        if value:
+            names.append(str(value))
+    eicon = find_glob_icon(*names, category)
+    if not eicon:
+        return
+    bdata, ext = eicon
+    entry_props["IconData"] = bdata
+    entry_props["IconType"] = ext
+
+
+def find_glob_icon(*names: str, category: str = "categories") -> tuple | None:
     if not LOAD_GLOB:
         return None
     icondirs = getattr(IconTheme, "icondirs", [])
@@ -306,12 +330,7 @@ def load_xdg_entry(de) -> dict[str, Any]:
         return {}
     icondata = props.get("IconData")
     if not icondata:
-        # try harder:
-        icondata = find_glob_icon(de, category="apps")
-        if icondata:
-            bdata, ext = icondata
-            props["IconData"] = bdata
-            props["IconType"] = ext
+        add_entry_icon(props, category="apps")
     return props
 
 
@@ -322,16 +341,11 @@ def load_xdg_menu(submenu) -> dict[str, Any]:
     ))
     icondata = submenu_data.get("IconData")
     if not icondata:
-        # try harder:
-        icondata = find_glob_icon(submenu_data, category="categories")
-        if icondata:
-            bdata, ext = icondata
-            submenu_data["IconData"] = bdata
-            submenu_data["IconType"] = ext
+        add_entry_icon(submenu_data, category="categories")
     entries_data = submenu_data.setdefault("Entries", {})
     from xdg.Menu import Menu, MenuEntry  # pylint: disable=import-outside-toplevel
 
-    def add_entries(entries):
+    def add_entries(entries) -> None:
         for i, entry in enumerate(entries):
             if isinstance(entry, MenuEntry):
                 de = entry.DesktopEntry
@@ -369,8 +383,8 @@ def remove_icons(menu_data):
     return filt
 
 
-def load_menu():
-    if not check_xdg():
+def load_menu() -> dict:
+    if not ENABLED or not check_xdg():
         return {}
     icon_util.large_icons.clear()
     start = monotonic()
@@ -391,7 +405,7 @@ def load_menu():
     return xdg_menu_data
 
 
-def load_xdg_menu_data():
+def load_xdg_menu_data() -> dict:
     try:
         from xdg.Menu import parse, Menu  # pylint: disable=import-outside-toplevel
     except ImportError:
@@ -399,8 +413,8 @@ def load_xdg_menu_data():
         if first_time("no-python-xdg"):
             log.warn("Warning: cannot use application menu data:")
             log.warn(" no python-xdg module")
-        return None
-    menu = None
+        return {}
+    menu: dict | None = None
     error = None
     # see ticket #2340,
     # invalid values for XDG_CONFIG_DIRS can cause problems,
@@ -462,7 +476,7 @@ def load_xdg_menu_data():
             log.error(" or an invalid system menu configuration")
             log.error(" for more information, please see:")
             log.error(" https://github.com/Xpra-org/xpra/issues/2174")
-        return None
+        return {}
     menu_data = {}
     entries = tuple(menu.getEntries())
     log(f"{menu}.getEntries()={entries}")
