@@ -1,5 +1,5 @@
 # This file is part of Xpra.
-# Copyright (C) 2013-2024 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2013 Antoine Martin <antoine@xpra.org>
 # Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
@@ -23,8 +23,10 @@ from xpra.os_util import (
 from xpra.util.io import umask_context
 from xpra.net.common import PacketType, is_request_allowed
 from xpra.net.socket_util import SOCKET_DIR_MODE, SOCKET_DIR_GROUP
+from xpra.server import features
 from xpra.server.core import ServerCore
-from xpra.server.control_command import ArgsControlCommand, ControlError
+from xpra.net.control.common import ArgsControlCommand, ControlError
+from xpra.auth.common import SessionData
 from xpra.util.child_reaper import getChildReaper
 from xpra.scripts.parsing import str_to_bool, MODE_ALIAS
 from xpra.scripts.config import make_defaults_struct, PROXY_START_OVERRIDABLE_OPTIONS, OPTION_TYPES
@@ -43,10 +45,11 @@ PROXY_SOCKET_TIMEOUT = envfloat("XPRA_PROXY_SOCKET_TIMEOUT", 0.1)
 PROXY_WS_TIMEOUT = envfloat("XPRA_PROXY_WS_TIMEOUT", 1.0)
 assert PROXY_SOCKET_TIMEOUT > 0, "invalid proxy socket timeout"
 CAN_STOP_PROXY = envbool("XPRA_CAN_STOP_PROXY", getuid() != 0 or WIN32)
-STOP_PROXY_SOCKET_TYPES = os.environ.get("XPRA_STOP_PROXY_SOCKET_TYPES", "socket,named-pipe").split(",")
+STOP_PROXY_SOCKET_TYPES = os.environ.get("XPRA_STOP_PROXY_SOCKET_TYPES", "named-pipe").split(",")
 STOP_PROXY_AUTH_SOCKET_TYPES = os.environ.get("XPRA_STOP_PROXY_AUTH_SOCKET_TYPES", "socket").split(",")
 # something (a thread lock?) doesn't allow us to use multiprocessing on MS Windows:
 PROXY_INSTANCE_THREADED = envbool("XPRA_PROXY_INSTANCE_THREADED", WIN32)
+PROXY_SSL_THREADED = envbool("XPRA_PROXY_SSL_THREADED", False)
 PROXY_CLEANUP_GRACE_PERIOD = envfloat("XPRA_PROXY_CLEANUP_GRACE_PERIOD", 0.5)
 
 MAX_CONCURRENT_CONNECTIONS = envint("XPRA_PROXY_MAX_CONCURRENT_CONNECTIONS", 200)
@@ -64,14 +67,86 @@ if WIN32:
 ENV_WHITELIST: list[str] = os.environ.get("XPRA_PROXY_ENV_WHITELIST", DEFAULT_ENV_WHITELIST).split(",")
 
 
-def get_socktype(proto):
+def get_socktype(proto) -> str:
     try:
         return proto._conn.socktype
     except AttributeError:
         return "unknown"
 
 
-class ProxyServer(ServerCore):
+def create_system_dir(sps: str) -> None:
+    if not POSIX or OSX or not sps:
+        return
+    xpra_group_id = get_group_id(SOCKET_DIR_GROUP)
+    if sps.startswith("/run/xpra") or sps.startswith("/var/run/xpra"):
+        # create the directory and verify its permissions
+        # which should have been set correctly by tmpfiles.d,
+        # but may have been set wrong if created by systemd's socket activation instead
+        d = sps.split("/xpra")[0] + "/xpra"
+        try:
+            if os.path.exists(d):
+                stat = os.stat(d)
+                mode = stat.st_mode
+                if (mode & SOCKET_DIR_MODE) != SOCKET_DIR_MODE:
+                    log.warn("Warning: invalid permissions on '%s' : %s", d, oct(mode))
+                    mode = mode | SOCKET_DIR_MODE
+                    log.warn(" changing to %s", oct(mode))
+                    os.chmod(d, mode)
+                # noinspection PyChainedComparisons
+                if xpra_group_id >= 0 and stat.st_gid != xpra_group_id:
+                    import grp
+                    group = grp.getgrgid(stat.st_gid)[0]
+                    log.warn("Warning: invalid group on '%s': %s", d, group)
+                    log.warn(" changing to '%s'", SOCKET_DIR_GROUP)
+                    os.lchown(d, stat.st_uid, xpra_group_id)
+            else:
+                log.info("creating '%s' with permissions %s and group '%s'",
+                         d, oct(SOCKET_DIR_MODE), SOCKET_DIR_GROUP)
+                with umask_context(0):
+                    os.mkdir(d, SOCKET_DIR_MODE)
+                stat = os.stat(d)
+                # noinspection PyChainedComparisons
+                if xpra_group_id >= 0 and stat.st_gid != xpra_group_id:
+                    os.lchown(d, stat.st_uid, xpra_group_id)
+            mode = os.stat(d).st_mode
+            log("%s permissions: %s", d, oct(mode))
+        except OSError as e:
+            log("create_system_dir()", exc_info=True)
+            log.error("Error: failed to create or change the permissions on '%s':", d)
+            log.estr(e)
+
+
+def get_proxy_env() -> dict[str, str]:
+    env = {k: v for k, v in os.environ.items() if k in ENV_WHITELIST or "*" in ENV_WHITELIST}
+    # env var to add to environment of subprocess:
+    extra_env_str = os.environ.get("XPRA_PROXY_START_ENV", "")
+    if extra_env_str:
+        extra_env = {}
+        for e in extra_env_str.split(os.path.pathsep):  # ie: "A=1:B=2"
+            parts = e.split("=", 1)
+            if len(parts) == 2:
+                extra_env[parts[0]] = parts[1]
+        log("extra_env(%s)=%s", extra_env_str, extra_env)
+        env.update(extra_env)
+    return env
+
+
+def get_proxy_server_base_classes() -> tuple[type, ...]:
+    classes: list[type] = [ServerCore]
+    if features.dbus:
+        from xpra.server.mixins.dbus import DbusServer
+        classes.append(DbusServer)
+    if features.http:
+        from xpra.server.mixins.http import HttpServer
+        classes.append(HttpServer)
+    return tuple(classes)
+
+
+SERVER_BASES = get_proxy_server_base_classes()
+ProxyServerBaseClass = type("ProxyServerBaseClass", SERVER_BASES, {})
+
+
+class ProxyServer(ProxyServerBaseClass):
     """
         This is the proxy server you can launch with "xpra proxy",
         once authenticated, it will dispatch the connection
@@ -81,7 +156,8 @@ class ProxyServer(ServerCore):
 
     def __init__(self):
         log("ProxyServer.__init__()")
-        super().__init__()
+        for bc in SERVER_BASES:
+            bc.__init__(self)
         self._max_connections = MAX_CONCURRENT_CONNECTIONS
         self._start_sessions = False
         self.session_type = "proxy"
@@ -90,7 +166,6 @@ class ProxyServer(ServerCore):
         # or even start them, so allow more time before timing out:
         self._accept_timeout += 10
         self.pings = 0
-        self.video_encoders = ()
         self._start_sessions = False
         # keep track of the proxy process instances
         # the display they're on and the message queue we can
@@ -98,70 +173,28 @@ class ProxyServer(ServerCore):
         self.instances = {}
         # connections used exclusively for requests:
         self._requests = set()
-        self.idle_add = GLib.idle_add
-        self.timeout_add = GLib.timeout_add
-        self.source_remove = GLib.source_remove
         self._socket_timeout = PROXY_SOCKET_TIMEOUT
         self._ws_timeout = PROXY_WS_TIMEOUT
 
     def init(self, opts) -> None:
         log("ProxyServer.init(%s)", opts)
         self.pings = int(opts.pings)
-        self.video_encoders = opts.proxy_video_encoders
         self._start_sessions = opts.proxy_start_sessions
-        super().init(opts)
+        for bc in SERVER_BASES:
+            bc.init(self, opts)
         # ensure we cache the platform info before intercepting SIGCHLD
         # as this will cause a fork and SIGCHLD to be emitted:
         from xpra.util.version import get_platform_info
         get_platform_info()
         self.child_reaper = getChildReaper()
-        self.create_system_dir(opts.system_proxy_socket)
-
-    def create_system_dir(self, sps) -> None:
-        if not POSIX or OSX or not sps:
-            return
-        xpra_group_id = get_group_id(SOCKET_DIR_GROUP)
-        if sps.startswith("/run/xpra") or sps.startswith("/var/run/xpra"):
-            # create the directory and verify its permissions
-            # which should have been set correctly by tmpfiles.d,
-            # but may have been set wrong if created by systemd's socket activation instead
-            d = sps.split("/xpra")[0] + "/xpra"
-            try:
-                if os.path.exists(d):
-                    stat = os.stat(d)
-                    mode = stat.st_mode
-                    if (mode & SOCKET_DIR_MODE) != SOCKET_DIR_MODE:
-                        log.warn("Warning: invalid permissions on '%s' : %s", d, oct(mode))
-                        mode = mode | SOCKET_DIR_MODE
-                        log.warn(" changing to %s", oct(mode))
-                        os.chmod(d, mode)
-                    # noinspection PyChainedComparisons
-                    if xpra_group_id >= 0 and stat.st_gid != xpra_group_id:
-                        import grp
-                        group = grp.getgrgid(stat.st_gid)[0]
-                        log.warn("Warning: invalid group on '%s': %s", d, group)
-                        log.warn(" changing to '%s'", SOCKET_DIR_GROUP)
-                        os.lchown(d, stat.st_uid, xpra_group_id)
-                else:
-                    log.info("creating '%s' with permissions %s and group '%s'",
-                             d, oct(SOCKET_DIR_MODE), SOCKET_DIR_GROUP)
-                    with umask_context(0):
-                        os.mkdir(d, SOCKET_DIR_MODE)
-                    stat = os.stat(d)
-                    # noinspection PyChainedComparisons
-                    if xpra_group_id >= 0 and stat.st_gid != xpra_group_id:
-                        os.lchown(d, stat.st_uid, xpra_group_id)
-                mode = os.stat(d).st_mode
-                log("%s permissions: %s", d, oct(mode))
-            except OSError as e:
-                log("create_system_dir()", exc_info=True)
-                log.error("Error: failed to create or change the permissions on '%s':", d)
-                log.estr(e)
+        create_system_dir(opts.system_proxy_socket)
 
     def init_control_commands(self) -> None:
         super().init_control_commands()
-        self.control_commands["stop"] = ArgsControlCommand("stop", "stops the proxy instance on the given display",
-                                                           self.handle_stop_command, min_args=1, max_args=1)
+        if features.control:
+            self.add_control_command("stop", ArgsControlCommand("stop",
+                                                                "stops the proxy instance on the given display",
+                                                                self.handle_stop_command, min_args=1, max_args=1))
 
     def install_signal_handlers(self, callback: Callable[[int], None]) -> None:
         from xpra.gtk.signals import register_SIGUSR_signals, register_os_signals
@@ -188,12 +221,6 @@ class ProxyServer(ServerCore):
     def get_server_mode(self) -> str:
         return "proxy"
 
-    def init_aliases(self) -> None:
-        """
-        It is a lot less confusing if proxy servers don't use packet aliases at all.
-        So we override the aliases initialization and skip it.
-        """
-
     def do_run(self) -> None:
         self.main_loop = GLib.MainLoop()
         self.main_loop.run()
@@ -244,7 +271,8 @@ class ProxyServer(ServerCore):
 
     def cleanup(self) -> None:
         self.stop_all_proxies()
-        super().cleanup()
+        for bc in SERVER_BASES:
+            bc.cleanup(self)
         start = monotonic()
         live = True
         log("cleanup() proxy instances: %s", self.instances)
@@ -274,7 +302,7 @@ class ProxyServer(ServerCore):
         if not protocol.is_closed():
             self.send_disconnect(protocol, ConnectionMessage.LOGIN_TIMEOUT)
 
-    def hello_oked(self, proto, c, auth_caps) -> None:
+    def hello_oked(self, proto, c: typedict, auth_caps: dict) -> None:
         if super().hello_oked(proto, c, auth_caps):
             # already handled in superclass
             return
@@ -282,18 +310,31 @@ class ProxyServer(ServerCore):
         request = c.strget("request")
 
         if request:
-            self.handle_hello_request(proto, request)
+            if not self.handle_hello_request(request, proto, c):
+                self.send_disconnect(proto, f"error: invalid request, {request!r} is not supported by the proxy server")
             return
         self.proxy_auth(proto, c, auth_caps)
 
-    def handle_hello_request(self, proto, request):
+    def handle_hello_request(self, request: str, proto, caps: typedict) -> bool:
         if request == "stop":
             self.handle_stop_request(proto)
-        else:
-            self.send_disconnect(proto, f"error: invalid request, {request!r} is not supported by the proxy server")
+            return True
+        if request == "id":
+            self.send_id_info(proto)
+            return True
+        if request == "info":
+            info = self.get_session_id_info()
+            log_file = os.environ.get("XPRA_SERVER_LOG", "")
+            info["log-file"] = log_file
+            self.do_send_info(proto, info)
+            return True
+        return False
 
-    def handle_stop_request(self, proto):
-        default_can_stop = CAN_STOP_PROXY and get_socktype(proto) in STOP_PROXY_SOCKET_TYPES and proto.authenticators
+    def handle_stop_request(self, proto) -> None:
+        socktype = get_socktype(proto)
+        default_can_stop = CAN_STOP_PROXY
+        default_can_stop |= socktype in STOP_PROXY_SOCKET_TYPES
+        default_can_stop |= (socktype in STOP_PROXY_AUTH_SOCKET_TYPES) and proto.authenticators
         if not is_request_allowed(proto, "stop", default_can_stop):
             msg = "`stop` requests are not enabled for this proxy server connection"
             log.warn(f"Warning: {msg}")
@@ -305,7 +346,7 @@ class ProxyServer(ServerCore):
         capabilities = self.make_hello(None)
         proto.send_now(("hello", capabilities))
 
-        def force_exit_request_client():
+        def force_exit_request_client() -> None:
             try:
                 self._requests.remove(proto)
             except KeyError:
@@ -313,9 +354,9 @@ class ProxyServer(ServerCore):
             if not proto.is_closed():
                 self.send_disconnect(proto, "timeout")
 
-        self.timeout_add(10 * 1000, force_exit_request_client)
+        GLib.timeout_add(10 * 1000, force_exit_request_client)
 
-    def proxy_auth(self, client_proto, c, auth_caps) -> None:
+    def proxy_auth(self, client_proto, c: typedict, auth_caps: dict) -> None:
         def disconnect(reason, *extras) -> None:
             log("disconnect(%s, %s)", reason, extras)
             self.send_disconnect(client_proto, reason, *extras)
@@ -333,7 +374,7 @@ class ProxyServer(ServerCore):
             log.error(" use 'none' to disable authentication")
             nosession("no sessions found")
             return
-        sessions = None
+        sessions : SessionData | None = None
         for authenticator in client_proto.authenticators:
             try:
                 auth_sessions = authenticator.get_sessions()
@@ -353,7 +394,7 @@ class ProxyServer(ServerCore):
             return
         self.proxy_session(client_proto, c, auth_caps, sessions)
 
-    def proxy_session(self, client_proto, c, auth_caps, sessions) -> None:
+    def proxy_session(self, client_proto, c: typedict, auth_caps: dict, sessions: SessionData) -> None:
         def disconnect(reason, *extras) -> None:
             log("disconnect(%s, %s)", reason, extras)
             self.send_disconnect(client_proto, reason, *extras)
@@ -395,7 +436,7 @@ class ProxyServer(ServerCore):
         proc = None
         socket_path = None
         display = None
-        sns = typedict(c.dictget("start-new-session", {}))
+        sns = c.dictget("start-new-session", {})
         authlog("proxy_session: displays=%s, start_sessions=%s, start-new-session=%s",
                 displays, self._start_sessions, sns)
         if not displays or sns:
@@ -407,7 +448,7 @@ class ProxyServer(ServerCore):
                 log("start_new_session%s=%s", (username, "..", uid, gid, sns, displays), (proc, socket_path, display))
             except Exception as e:
                 log("start_server_subprocess failed", exc_info=True)
-                log.error("Error: failed to start server subprocess:")
+                log.error("Error: failed to start new session subprocess:")
                 log.estr(e)
                 nosession("failed to start a new session")
                 return
@@ -427,7 +468,7 @@ class ProxyServer(ServerCore):
                         return
             else:
                 if len(displays) != 1:
-                    nosession("please specify a display, more than one is available: " + csv(displays))
+                    nosession("please specify a display, more than one is available: " + csv(sorted(displays)))
                     return
                 display = displays[0]
 
@@ -441,7 +482,7 @@ class ProxyServer(ServerCore):
             if socket_path:
                 hello["socket-path"] = socket_path
             # echo mode if present:
-            mode = sns.strget("mode")
+            mode = str(sns.get("mode", ""))
             if mode:
                 hello["mode"] = mode
             client_proto.send_now(("hello", hello))
@@ -483,7 +524,7 @@ class ProxyServer(ServerCore):
             return
         log("server connection=%s", server_conn)
 
-        cipher = cipher_mode = None
+        cipher = cipher_mode = ""
         encryption_key = b""
         if auth_caps:
             cipher = auth_caps.get("cipher")
@@ -493,7 +534,7 @@ class ProxyServer(ServerCore):
                 encryption_key = self.get_encryption_key(client_proto.authenticators, client_proto.keyfile)
 
         use_thread = PROXY_INSTANCE_THREADED
-        if not use_thread:
+        if not use_thread and PROXY_SSL_THREADED:
             client_socktype = get_socktype(client_proto)
             server_socktype = disp_desc["type"]
             if client_socktype in ("ssl", "wss", "ssh"):
@@ -506,7 +547,7 @@ class ProxyServer(ServerCore):
             if env_options:
                 log.warn("environment options are ignored in threaded mode")
             from xpra.server.proxy.instance_thread import ProxyInstanceThread
-            pit = ProxyInstanceThread(session_options, self.video_encoders, self.pings,
+            pit = ProxyInstanceThread(session_options, self.pings,
                                       client_proto, server_conn,
                                       disp_desc, cipher, cipher_mode, encryption_key, c)
             pit.stopped = self.reap
@@ -521,7 +562,7 @@ class ProxyServer(ServerCore):
             client_conn = None
             try:
                 # no other packets should be arriving until the proxy instance responds to the initial hello packet
-                def unexpected_packet(packet):
+                def unexpected_packet(packet) -> None:
                     if packet:
                         log.warn("Warning: received an unexpected packet")
                         log.warn(" from the proxy connection %s:", client_proto)
@@ -541,7 +582,7 @@ class ProxyServer(ServerCore):
                 client_conn.set_active(True)
                 from xpra.server.proxy.instance_process import ProxyInstanceProcess
                 process = ProxyInstanceProcess(uid, gid, env_options, session_options, self._socket_dir,
-                                               self.video_encoders, self.pings,
+                                               self.pings,
                                                client_conn, disp_desc, client_state,
                                                cipher, cipher_mode, encryption_key, server_conn, c, message_queue)
                 log("starting %s from pid=%s", process, os.getpid())
@@ -553,11 +594,11 @@ class ProxyServer(ServerCore):
                 # when this process dies, run reap to update our list of proxy instances:
                 self.child_reaper.add_process(popen, f"xpra-proxy-{display}",
                                               "xpra-proxy-instance", True, True, self.reap)
-            except Exception as e:
+            except Exception as pie:
                 log("start_proxy_process() failed", exc_info=True)
                 log.error("Error starting proxy instance process:")
-                log.estr(e)
-                message_queue.put(f"error: {e}")
+                log.estr(pie)
+                message_queue.put(f"error: {pie}")
                 message_queue.put("stop")
             finally:
                 # now we can close our handle on the connection:
@@ -571,9 +612,9 @@ class ProxyServer(ServerCore):
         start_thread(start_proxy_process, f"start_proxy({client_proto})")
 
     def start_new_session(self, username: str, _password, uid: int, gid: int,
-                          new_session_dict=None, displays=()) -> tuple[Any, str, str]:
-        log("start_new_session%s", (username, "..", uid, gid, new_session_dict, displays))
-        sns = typedict(new_session_dict or {})
+                          sess_options: dict, displays=()) -> tuple[Any, str, str]:
+        log("start_new_session%s", (username, "..", uid, gid, sess_options, displays))
+        sns = typedict(sess_options or {})
         mode = sns.strget("mode", "start")
         mode = MODE_ALIAS.get(mode, mode)
         if mode not in ("seamless", "desktop", "shadow", "monitor", "expand"):
@@ -597,13 +638,13 @@ class ProxyServer(ServerCore):
                 continue
             vt = OPTION_TYPES[k]
             try:
-                if vt == str:
+                if vt is str:
                     v = bytestostr(v)
-                elif vt == bool:
+                elif vt is bool:
                     v = str_to_bool(v)
-                elif vt == int:
+                elif vt is int:
                     v = int(v)
-                elif vt == list:
+                elif vt is list:
                     v = list(bytestostr(x) for x in v)
             except ValueError:
                 log("start_new_session: override option %s", k, exc_info=True)
@@ -621,7 +662,7 @@ class ProxyServer(ServerCore):
                 log("start override: %24s=%-24s (invalid, unchanged)", k, v)
         opts.attach = False
         opts.start_via_proxy = False
-        env = self.get_proxy_env()
+        env = get_proxy_env()
         cwd = None
         if uid > 0:
             cwd = get_home_for_uid(uid) or None
@@ -639,20 +680,6 @@ class ProxyServer(ServerCore):
         log("start_new_session(..) pid=%s, socket_path=%s, display=%s, ", proc.pid, socket_path, display)
         return proc, socket_path, display
 
-    def get_proxy_env(self) -> dict[str, str]:
-        env = {k: v for k, v in os.environ.items() if k in ENV_WHITELIST or "*" in ENV_WHITELIST}
-        # env var to add to environment of subprocess:
-        extra_env_str = os.environ.get("XPRA_PROXY_START_ENV", "")
-        if extra_env_str:
-            extra_env = {}
-            for e in extra_env_str.split(os.path.pathsep):  # ie: "A=1:B=2"
-                parts = e.split("=", 1)
-                if len(parts) == 2:
-                    extra_env[parts[0]] = parts[1]
-            log("extra_env(%s)=%s", extra_env_str, extra_env)
-            env.update(extra_env)
-        return env
-
     def reap(self, *args) -> None:
         log("reap%s", args)
         dead = []
