@@ -1,5 +1,5 @@
 # This file is part of Xpra.
-# Copyright (C) 2016-2024 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2016 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -15,8 +15,8 @@ from xpra.gtk.error import xlog
 from xpra.x11.common import get_wm_name
 from xpra.x11.models.model_stub import WindowModelStub
 from xpra.x11.bindings.window import X11WindowBindings
-from xpra.x11.gtk_x11.damage import WindowDamageHandler
-from xpra.x11.gtk3.bindings import add_event_receiver, remove_event_receiver
+from xpra.x11.gtk.damage import WindowDamageHandler
+from xpra.x11.gtk.bindings import add_event_receiver, remove_event_receiver
 from xpra.x11.bindings.randr import RandRBindings
 from xpra.log import Logger
 
@@ -28,7 +28,6 @@ geomlog = Logger("server", "window", "geometry")
 iconlog = Logger("icon")
 
 GObject = gi_import("GObject")
-Gdk = gi_import("Gdk")
 GLib = gi_import("GLib")
 
 
@@ -39,8 +38,8 @@ class DesktopModelBase(WindowModelStub, WindowDamageHandler):
         "resized": no_arg_signal,
         "client-contents-changed": one_arg_signal,
         "motion": one_arg_signal,
-        "xpra-motion-event": one_arg_signal,
-        "xpra-property-notify-event": one_arg_signal,
+        "x11-motion-event": one_arg_signal,
+        "x11-property-notify-event": one_arg_signal,
     }
 
     __gproperties__ = {
@@ -88,9 +87,8 @@ class DesktopModelBase(WindowModelStub, WindowDamageHandler):
     _dynamic_property_names = ["size-hints", "title", "icons"]
 
     def __init__(self):
-        screen = Gdk.Screen.get_default()
-        root = screen.get_root_window()
-        WindowDamageHandler.__init__(self, root.get_xid())
+        root_xid = X11Window.get_root_xid()
+        WindowDamageHandler.__init__(self, root_xid)
         WindowModelStub.__init__(self)
         self.update_wm_name()
         self.update_icon()
@@ -105,10 +103,12 @@ class DesktopModelBase(WindowModelStub, WindowDamageHandler):
         self._managed = True
         self._setup_done = True
         # listen for property changes on the root window:
-        add_event_receiver(self.xid, self)
+        # (monitor mode can have up to 16 monitors, so we may have many event listeners
+        #  so raise the limit to 20)
+        add_event_receiver(self.xid, self, 20)
 
-    def do_xpra_property_notify_event(self, event) -> None:
-        eventlog(f"do_xpra_property_notify_event: {event.atom}")
+    def do_x11_property_notify_event(self, event) -> None:
+        eventlog(f"do_x11_property_notify_event: {event.atom}")
         # update the wm-name (and therefore the window's "title")
         # whenever this property changes:
         if str(event.atom) == "_NET_SUPPORTING_WM_CHECK":
@@ -131,7 +131,6 @@ class DesktopModelBase(WindowModelStub, WindowDamageHandler):
         return self._updateprop("wm-name", wm_name)
 
     def update_icon(self) -> bool:
-        icons = None
         wm_name = self.get_property("wm-name")
         if not wm_name:
             return False
@@ -192,13 +191,13 @@ class DesktopModelBase(WindowModelStub, WindowDamageHandler):
             return False
         return GObject.GObject.get_property(self, prop)
 
-    def do_xpra_damage_event(self, event) -> None:
+    def do_x11_damage_event(self, event) -> None:
         self.emit("client-contents-changed", event)
 
-    def do_xpra_motion_event(self, event) -> None:
+    def do_x11_motion_event(self, event) -> None:
         self.emit("motion", event)
 
-    def resize(self, w: int, h: int):
+    def resize(self, w: int, h: int) -> None:
         geomlog("resize(%i, %i)", w, h)
         if not RandR.has_randr():
             geomlog.error("Error: cannot honour resize request,")
@@ -210,10 +209,10 @@ class DesktopModelBase(WindowModelStub, WindowDamageHandler):
         if not self.resize_timer:
             self.resize_timer = GLib.timeout_add(250, self.do_resize)
 
-    def do_resize(self):
+    def do_resize(self) -> None:
         raise NotImplementedError
 
-    def cancel_resize_timer(self):
+    def cancel_resize_timer(self) -> None:
         rt = self.resize_timer
         if rt:
             self.resize_timer = 0
