@@ -1,5 +1,5 @@
 # This file is part of Xpra.
-# Copyright (C) 2019-2024 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2019 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -10,9 +10,11 @@ from collections.abc import Iterable, Sequence
 
 from xpra.os_util import gi_import
 from xpra.util.env import envbool
-from xpra.gtk.error import xsync, xswallow
+from xpra.common import noop
+from xpra.gtk.error import xsync
 from xpra.gtk.gobject import n_arg_signal, one_arg_signal
 from xpra.gtk.util import get_default_root_window
+from xpra.x11.gtk.common import get_wininfo
 from xpra.x11.gtk.native_window import GDKX11Window
 from xpra.x11.gtk.bindings import (
     add_event_receiver,
@@ -24,7 +26,6 @@ from xpra.gtk.error import XError
 from xpra.clipboard.core import ClipboardProxyCore, TEXT_TARGETS, must_discard, must_discard_extra, ClipboardCallback
 from xpra.clipboard.timeout import ClipboardTimeoutHelper, CONVERT_TIMEOUT
 from xpra.x11.bindings.window import constants, PropertyError, X11WindowBindings
-from xpra.x11.bindings.res import ResBindings
 from xpra.util.env import first_time
 from xpra.util.str_fn import csv, Ellipsizer, repr_ellipsized, bytestostr, memoryview_to_bytes
 from xpra.log import Logger
@@ -34,9 +35,6 @@ Gdk = gi_import("Gdk")
 GLib = gi_import("GLib")
 
 X11Window = X11WindowBindings()
-XRes = ResBindings()
-if not XRes.check_xres():
-    XRes = None
 
 log = Logger("x11", "clipboard")
 
@@ -88,6 +86,15 @@ TRANSLATED_TARGETS = parse_translated_targets(os.environ.get(
 )
 log("TRANSLATED_TARGETS=%s", TRANSLATED_TARGETS)
 
+IGNORED_PROPERTIES = (
+    "_NET_WM_NAME", "WM_NAME", "_NET_WM_ICON_NAME", "WM_ICON_NAME",
+    "WM_PROTOCOLS", "WM_NORMAL_HINTS", "WM_CLIENT_MACHINE", "WM_LOCALE_NAME",
+    "_NET_WM_PID", "WM_CLIENT_LEADER", "_NET_WM_USER_TIME_WINDOW",
+)
+IGNORED_MESSAGES = (
+    "_GTK_LOAD_ICONTHEMES",
+)
+
 
 def xatoms_to_strings(data: bytes) -> Sequence[str]:
     length = len(data)
@@ -96,8 +103,7 @@ def xatoms_to_strings(data: bytes) -> Sequence[str]:
     natoms = length // sizeof_long
     atoms = struct.unpack(b"@" + b"L" * natoms, data)
     with xsync:
-        return tuple(bytestostr(name) for name in (X11Window.XGetAtomName(atom)
-                                                   for atom in atoms if atom) if name is not None)
+        return tuple(name for name in (X11Window.get_atom_name(atom) for atom in atoms if atom) if name)
 
 
 def strings_to_xatoms(data: Iterable[str]) -> bytes:
@@ -106,39 +112,6 @@ def strings_to_xatoms(data: Iterable[str]) -> bytes:
     return struct.pack(b"@" + b"L" * len(atom_array), *atom_array)
 
 
-def get_wintitle(xid) -> str:
-    with xswallow:
-        data = X11Window.XGetWindowProperty(xid, "WM_NAME", "STRING")
-        if data:
-            return data.decode("latin1")
-    with xswallow:
-        data = X11Window.XGetWindowProperty(xid, "_NET_WM_NAME", "UTF8_STRING")
-        if data:
-            return data.decode("utf8")
-    return ""
-
-
-def get_wininfo(xid) -> str:
-    wininfo = [f"xid={xid:x}"]
-    if XRes:
-        with xswallow:
-            pid = XRes.get_pid(xid)
-            if pid:
-                wininfo.append(f"pid={pid}")
-    title = get_wintitle(xid)
-    if title:
-        wininfo.insert(0, repr(title))
-        return csv(wininfo)
-    while xid:
-        title = get_wintitle(xid)
-        if title:
-            wininfo.append(f"child of {title!r}")
-            return csv(wininfo)
-        with xswallow:
-            xid = X11Window.getParent(xid)
-    return csv(wininfo)
-
-
 class ClipboardProxy(ClipboardProxyCore, GObject.GObject):
     __gsignals__ = {
         "x11-client-message-event": one_arg_signal,
@@ -330,16 +303,15 @@ class ClipboardProxy(ClipboardProxyCore, GObject.GObject):
 
         req_target = target
         if self.targets and target not in self.targets:
-            if first_time(f"client-{wininfo}-invalidtarget-{target}"):
-                log_fn = log.info
-            else:
-                log_fn = log.debug
-            log_fn("client %s is requesting an unknown target: '%s'", wininfo, target)
             translated_targets = TRANSLATED_TARGETS.get(target, ())
             can_translate = tuple(x for x in translated_targets if x in self.targets)
+            log_fn = log.debug
+            if not can_translate and first_time(f"client-{wininfo}-invalidtarget-{target}"):
+                log_fn = log.info
+            log_fn(f"client {wininfo} is requesting an unknown target: {target!r}")
             if can_translate:
                 req_target = can_translate[0]
-                log_fn(" using '%s' instead", req_target)
+                log_fn(f" using {req_target!r} instead")
             else:
                 log_fn(" valid targets: %s", csv(self.targets))
                 if must_discard_extra(target):
@@ -434,11 +406,11 @@ class ClipboardProxy(ClipboardProxyCore, GObject.GObject):
         # we need the targets, and the target data for greedy clients:
 
         def send_token_with_targets() -> None:
-            token_data = (self.targets,)
+            token_data = (self.targets, )
             self._have_token = False
             self.emit("send-clipboard-token", token_data)
 
-        def with_targets(otargets) -> None:
+        def with_targets(otargets: Sequence[str]) -> None:
             if not self._greedy_client:
                 send_token_with_targets()
                 return
@@ -568,7 +540,6 @@ class ClipboardProxy(ClipboardProxyCore, GObject.GObject):
         try:
             with xsync:
                 dtype, dformat = X11Window.GetWindowPropertyType(self.xid, event.atom, True)
-                dtype = bytestostr(dtype)
                 data = X11Window.XGetWindowProperty(self.xid, event.atom, dtype, buffer_size=MAX_DATA_SIZE, incr=True)
                 # all the code below deals with INCRemental transfers:
                 if dtype == "INCR" and not self.incr_data_size:
@@ -632,9 +603,8 @@ class ClipboardProxy(ClipboardProxyCore, GObject.GObject):
             GLib.source_remove(idt)
 
     def incr_data_timeout(self) -> None:
-        self.incr_data_timer = 0
         log.warn("Warning: incremental data timeout")
-        self.incr_data = None
+        self.reset_incr_data()
 
 
 GObject.type_register(ClipboardProxy)
@@ -651,7 +621,7 @@ class X11Clipboard(ClipboardTimeoutHelper, GObject.GObject):
         "x11-xfixes-selection-notify-event": one_arg_signal,
     }
 
-    def __init__(self, send_packet_cb, progress_cb=None, **kwargs):
+    def __init__(self, send_packet_cb, progress_cb=noop, **kwargs):
         GObject.GObject.__init__(self)
         self.init_window()
         init_x11_filter()
@@ -723,20 +693,16 @@ class X11Clipboard(ClipboardTimeoutHelper, GObject.GObject):
 
     def do_x11_client_message_event(self, event) -> None:
         message_type = event.message_type
-        if message_type == "_GTK_LOAD_ICONTHEMES":
+        if message_type in IGNORED_MESSAGES:
             log("ignored clipboard client message: %s", message_type)
             return
         log.info(f"Unexpected X11 message received by clipboard window {event.window:x}")
         log.info(f" {event}")
 
     def do_x11_property_notify_event(self, event) -> None:
-        if event.atom in (
-                "_NET_WM_NAME", "WM_NAME", "_NET_WM_ICON_NAME", "WM_ICON_NAME",
-                "WM_PROTOCOLS", "WM_NORMAL_HINTS", "WM_CLIENT_MACHINE", "WM_LOCALE_NAME",
-                "_NET_WM_PID", "WM_CLIENT_LEADER", "_NET_WM_USER_TIME_WINDOW",
-        ):
+        if event.atom in IGNORED_PROPERTIES:
             # these properties are populated by GTK when we create the window,
-            # no need to log them:
+            # no need to log them
             return
         log("do_x11_property_notify_event(%s)", event)
         # ie: atom=PRIMARY-TARGETS
